//
// Checkers images for the Fast Light Tool Kit (FLTK).
//
// Copyright 1998-2022 by Bill Spitzak and others.
//
// This library is free software. Distribution and use rights are outlined in
// the file "COPYING" which should have been included with this file.  If this
// file is missing or damaged, see the license at:
//
//     https://www.fltk.org/COPYING.php
//
// Please see the following page on how to report bugs and issues:
//
//     https://www.fltk.org/bugs.php
//

// generated by Fast Light User Interface Designer (fluid) version 1.0404

#ifndef checkers_pieces_h
#define checkers_pieces_h
#include <FL/Fl.H>
extern unsigned char pixmaps_black_checker_png[6118];
extern unsigned char pixmaps_white_checker_png[6536];
extern unsigned char pixmaps_black_checker_king_png[7322];
extern unsigned char pixmaps_white_checker_king_png[7548];
#endif
